==> odoo_19.2a1.20260202.zip: odoo-19.2a1.post20260202/odoo/addons/base/tests/test_ir_module.py <==
from odoo.tests.common import tagged, TransactionCase
from odoo.tools import mute_logger


@tagged('at_install', '-post_install')  # LEGACY at_install
class IrModuleCase(TransactionCase):
    @mute_logger("odoo.modules.module")
    def test_missing_module_icon(self):
        module = self.env["ir.module.module"].create({"name": "missing"})
        base = self.env["ir.module.module"].search([("name", "=", "base")])
        self.assertEqual(base.icon_image, module.icon_image)

    @mute_logger("odoo.modules.module")
    def test_new_module_icon(self):
        module = self.env["ir.module.module"].new({"name": "missing"})
        self.assertFalse(module.icon_image)

    @mute_logger("odoo.modules.module")
    def test_module_wrong_icon(self):
        module = self.env["ir.module.module"].create(
            {"name": "wrong_icon", "icon": "/not/valid.png"}
        )
        self.assertFalse(module.icon_image)

    @mute_logger("odoo.modules.module")
    def test_falsy_res_id(self):
        module = self.env["ir.module.module"].create(
            {"name": "get_views_test", "state": "installed"},
        )
        report = self.env['ir.actions.report'].create({
            'name': 'good_data_report',
            'report_name': 'web_studio.test_duplicate_foo',
            'model': 'res.users',
        })
        self.env["ir.model.data"].create({
            "module": "get_views_test",
            "name": "bad_data",
            "model": "ir.actions.report",
            "res_id": 0,
        })
        self.env["ir.model.data"].create({
            "module": "get_views_test",
            "name": "good_data",
            "model": "ir.actions.report",
            "res_id": report.id,
        })
        self.assertEqual(module.reports_by_module, "good_data_report")
        self.assertEqual(module.state, "installed")
        module.module_uninstall()
        self.assertEqual(module.state, "uninstalled")

==> odoo_19.3a1.20260226.zip: odoo-19.3a1.post20260226/odoo/addons/base/tests/test_ir_module.py <==
from odoo.exceptions import ValidationError
from odoo.tests.common import tagged, TransactionCase
from odoo.tools import mute_logger


@tagged('at_install', '-post_install')  # LEGACY at_install
class IrModuleCase(TransactionCase):
    @mute_logger("odoo.modules.module")
    def test_missing_module_icon(self):
        module = self.env["ir.module.module"].create({"name": "missing"})
        base = self.env["ir.module.module"].search([("name", "=", "base")])
        self.assertEqual(base.icon_image, module.icon_image)

    @mute_logger("odoo.modules.module")
    def test_new_module_icon(self):
        module = self.env["ir.module.module"].new({"name": "missing"})
        self.assertFalse(module.icon_image)

    @mute_logger("odoo.modules.module")
    def test_module_wrong_icon(self):
        module = self.env["ir.module.module"].create(
            {"name": "wrong_icon", "icon": "/not/valid.png"}
        )
        self.assertFalse(module.icon_image)

    @mute_logger("odoo.modules.module")
    def test_falsy_res_id(self):
        module = self.env["ir.module.module"].create(
            {"name": "get_views_test", "state": "installed"},
        )
        report = self.env['ir.actions.report'].create({
            'name': 'good_data_report',
            'report_name': 'web_studio.test_duplicate_foo',
            'model': 'res.users',
        })
        self.env["ir.model.data"].create({
            "module": "get_views_test",
            "name": "bad_data",
            "model": "ir.actions.report",
            "res_id": 0,
        })
        self.env["ir.model.data"].create({
            "module": "get_views_test",
            "name": "good_data",
            "model": "ir.actions.report",
            "res_id": report.id,
        })
        self.assertEqual(module.reports_by_module, "good_data_report")
        self.assertEqual(module.state, "installed")
        module.module_uninstall()
        self.assertEqual(module.state, "uninstalled")


@tagged('at_install', '-post_install')
class TestModuleCategory(TransactionCase):

    def test_parent_circular_dependencies(self):
        Cats = self.env['ir.module.category']

        def create(name, **kw):
            return Cats.create(dict(kw, name=name))

        category_a = create('A', parent_id=False)
        category_b = create('B', parent_id=category_a.id)
        category_c = create('C', parent_id=category_b.id)

        with self.assertRaises(ValidationError):
            category_a.write({'parent_id': category_c.id})
        with self.assertRaises(ValidationError):
            category_b.write({'parent_id': category_b.id})
